"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const path = require("path");
const exit_1 = require("exit");
process.on('message', (options) => {
    const parentPipe = (0, fs_extra_1.createWriteStream)('', { fd: 3 });
    parentPipe.setDefaultEncoding('utf-8');
    void invokeFunction(options)
        .then((result) => {
        parentPipe.write(JSON.stringify({ result }));
    })
        .catch((error) => {
        let plainError = error;
        if (typeof error === 'object') {
            plainError = Object.getOwnPropertyNames(error).reduce((acc, key) => {
                acc[key] = error[key];
                return acc;
            }, {});
        }
        parentPipe.write(JSON.stringify({ error: plainError }));
    })
        .then(() => {
        (0, exit_1.default)(0);
    });
});
const invokeFunction = (options) => __awaiter(void 0, void 0, void 0, function* () {
    if (options.packageFolder) {
        const p = path.resolve(options.packageFolder);
        if (!(0, fs_extra_1.existsSync)(p)) {
            throw new Error(`Lambda package folder ${options.packageFolder} does not exist`);
        }
        process.chdir(p);
    }
    else {
        throw new Error(`Invalid lambda invoke request. No package folder specified.`);
    }
    if (!options.handler) {
        throw new Error(`Invalid lambda invoke request. No handler specified.`);
    }
    const lambdaHandler = yield loadHandler(options.packageFolder, options.handler);
    const event = JSON.parse(options.event);
    const lambdaMockContext = Object.assign({ functionName: 'mock-function-name', functionVersion: '1', invokedFunctionArn: 'mock-function-arn', memoryLimitInMB: '128', awsRequestId: 'LAMBDA_INVOKE', logGroupName: 'LAMBDA_INVOKE', logStreamName: 'LAMBDA_INVOKE', callbackWaitsForEmptyEventLoop: true }, options.context);
    return new Promise((resolve, reject) => {
        const callback = (error, response) => {
            if (error) {
                reject(error);
            }
            else {
                resolve(response);
            }
        };
        try {
            const lambdaPromise = lambdaHandler(event, lambdaMockContext, callback);
            if (typeof lambdaPromise === 'object' && typeof lambdaPromise.then === 'function') {
                resolve(lambdaPromise);
            }
        }
        catch (e) {
            reject(e);
        }
    });
});
// handler is a string like 'path/to/handler.func'
// eslint-disable-next-line @typescript-eslint/ban-types
const loadHandler = (root, handler) => __awaiter(void 0, void 0, void 0, function* () {
    const handlerParts = path.parse(handler);
    try {
        const handler = yield Promise.resolve().then(() => require(path.join(root, handlerParts.dir, handlerParts.name)));
        const handlerFuncName = handlerParts.ext.replace('.', '');
        const handlerFunc = handler === null || handler === void 0 ? void 0 : handler[handlerFuncName];
        if (typeof handlerFunc !== 'function') {
            throw new Error(`Lambda handler ${handlerParts.name} has no exported function named ${handlerFuncName}`);
        }
        return handlerFunc;
    }
    catch (err) {
        throw new Error(`Could not load lambda handler function due to ${err}`);
    }
});
//# sourceMappingURL=execute.js.map